"""Top-level package for Altair Express."""

__author__ = """Dylan Wootton"""
__email__ = 'dwootton@mit.edu'
__version__ = '0.1.24'

from .jointplot import *
from .pairplot import *
from .distributional import *
from .relational import *
from .data_table import *
from .interactions import *